import React, { useEffect, useRef, useCallback } from 'react';
import { createResolver, DeferTiming } from '@shopify/async';
import { IntersectionObserver } from '@shopify/react-intersection-observer';
import { OnIdle, useIdleCallback } from '@shopify/react-idle';
import { Hydrator, useHydrationManager } from '@shopify/react-hydrate';
import { useAsync } from "./hooks.mjs";
import { AssetTiming } from "./types.mjs";
export function createAsyncComponent(_ref) {
  var id = _ref.id,
      load = _ref.load,
      defer = _ref.defer,
      deferHydration = _ref.deferHydration,
      displayName = _ref.displayName,
      _ref$renderLoading = _ref.renderLoading,
      renderLoading = _ref$renderLoading === void 0 ? noopRender : _ref$renderLoading,
      _ref$renderError = _ref.renderError,
      renderError = _ref$renderError === void 0 ? defaultRenderError : _ref$renderError,
      _ref$usePreload = _ref.usePreload,
      useCustomPreload = _ref$usePreload === void 0 ? noopUse : _ref$usePreload,
      _ref$usePrefetch = _ref.usePrefetch,
      useCustomPrefetch = _ref$usePrefetch === void 0 ? noopUse : _ref$usePrefetch,
      _ref$useKeepFresh = _ref.useKeepFresh,
      useCustomKeepFresh = _ref$useKeepFresh === void 0 ? noopUse : _ref$useKeepFresh;
  var resolver = createResolver({
    id: id,
    load: load
  });
  var componentName = displayName || displayNameFromId(resolver.id);
  var deferred = defer != null;
  var progressivelyHydrated = deferHydration != null;
  var scriptTiming = deferred || progressivelyHydrated ? AssetTiming.CurrentPage : AssetTiming.Immediate;
  var stylesTiming = deferred ? AssetTiming.CurrentPage : AssetTiming.Immediate;

  function Async(props) {
    var _useAsync = useAsync(resolver, {
      scripts: scriptTiming,
      styles: stylesTiming,
      immediate: !deferred
    }),
        Component = _useAsync.resolved,
        load = _useAsync.load,
        loading = _useAsync.loading,
        error = _useAsync.error;

    var _useRef = useRef(useHydrationManager().hydrated),
        startedHydrated = _useRef.current;

    if (error) {
      return renderError(error);
    }

    var loadingMarkup = null;

    if (progressivelyHydrated && !startedHydrated) {
      loadingMarkup = /*#__PURE__*/React.createElement(Loader, {
        defer: deferHydration,
        load: load,
        props: props
      });
    } else if (loading) {
      loadingMarkup = /*#__PURE__*/React.createElement(Loader, {
        defer: defer,
        load: load,
        props: props
      });
    }

    var contentMarkup = null;
    var rendered = Component ? /*#__PURE__*/React.createElement(Component, props) : null;

    if (progressivelyHydrated && !startedHydrated) {
      contentMarkup = rendered ? /*#__PURE__*/React.createElement(Hydrator, {
        id: resolver.id
      }, rendered) : /*#__PURE__*/React.createElement(Hydrator, {
        id: resolver.id
      });
    } else if (loading) {
      contentMarkup = renderLoading(props);
    } else {
      contentMarkup = rendered;
    }

    return /*#__PURE__*/React.createElement(React.Fragment, null, contentMarkup, loadingMarkup);
  }

  Async.displayName = "Async(".concat(componentName, ")");

  function usePreload(props) {
    var _useAsync2 = useAsync(resolver, {
      assets: AssetTiming.NextPage
    }),
        load = _useAsync2.load;

    var customPreload = useCustomPreload(props);
    return useCallback(function () {
      load();

      if (customPreload) {
        customPreload();
      }
    }, [load, customPreload]);
  }

  function usePrefetch(props) {
    var _useAsync3 = useAsync(resolver, {
      assets: AssetTiming.NextPage
    }),
        load = _useAsync3.load;

    var customPrefetch = useCustomPrefetch(props);
    return useCallback(function () {
      load();

      if (customPrefetch) {
        customPrefetch();
      }
    }, [load, customPrefetch]);
  }

  function useKeepFresh(props) {
    var _useAsync4 = useAsync(resolver, {
      assets: AssetTiming.NextPage
    }),
        load = _useAsync4.load;

    var customKeepFresh = useCustomKeepFresh(props);
    return useCallback(function () {
      load();

      if (customKeepFresh) {
        customKeepFresh();
      }
    }, [load, customKeepFresh]);
  }

  function Preload(options) {
    useIdleCallback(usePreload(options));
    return null;
  }

  Preload.displayName = "Async.Preload(".concat(displayName, ")");

  function Prefetch(options) {
    var prefetch = usePrefetch(options);
    useEffect(function () {
      prefetch();
    }, [prefetch]);
    return null;
  }

  Prefetch.displayName = "Async.Prefetch(".concat(displayName, ")");

  function KeepFresh(options) {
    useIdleCallback(useKeepFresh(options));
    return null;
  }

  KeepFresh.displayName = "Async.KeepFresh(".concat(displayName, ")");
  var FinalComponent = Async;
  Reflect.defineProperty(FinalComponent, 'resolver', {
    value: resolver,
    writable: false
  });
  Reflect.defineProperty(FinalComponent, 'Preload', {
    value: Preload,
    writable: false
  });
  Reflect.defineProperty(FinalComponent, 'Prefetch', {
    value: Prefetch,
    writable: false
  });
  Reflect.defineProperty(FinalComponent, 'KeepFresh', {
    value: KeepFresh,
    writable: false
  });
  Reflect.defineProperty(FinalComponent, 'usePreload', {
    value: usePreload,
    writable: false
  });
  Reflect.defineProperty(FinalComponent, 'usePrefetch', {
    value: usePrefetch,
    writable: false
  });
  Reflect.defineProperty(FinalComponent, 'useKeepFresh', {
    value: useKeepFresh,
    writable: false
  });
  return FinalComponent;
}

function noopUse() {
  return function () {};
}

function noopRender() {
  return null;
}

var DEFAULT_DISPLAY_NAME = 'Component';
var FILENAME_REGEX = /([^/]*)\.\w+$/;

function displayNameFromId(id) {
  if (!id) {
    return DEFAULT_DISPLAY_NAME;
  }

  var match = FILENAME_REGEX.exec(id);
  return match ? match[1] : DEFAULT_DISPLAY_NAME;
}

function defaultRenderError(error) {
  if (error) {
    throw error;
  }

  return null;
}

function Loader(_ref2) {
  var defer = _ref2.defer,
      load = _ref2.load,
      props = _ref2.props;
  var handleIntersection = useCallback(function (_ref3) {
    var _ref3$isIntersecting = _ref3.isIntersecting,
        isIntersecting = _ref3$isIntersecting === void 0 ? true : _ref3$isIntersecting;

    if (isIntersecting) {
      load();
    }
  }, [load]);
  useEffect(function () {
    if (defer == null || defer === DeferTiming.Mount) {
      load();
    } else if (typeof defer === 'function' && defer(props)) {
      load();
    }
  }, [defer, load, props]);

  if (typeof defer === 'function') {
    return null;
  }

  switch (defer) {
    case DeferTiming.Idle:
      return /*#__PURE__*/React.createElement(OnIdle, {
        perform: load
      });

    case DeferTiming.InViewport:
      return /*#__PURE__*/React.createElement(IntersectionObserver, {
        threshold: 0,
        onIntersectionChange: handleIntersection
      });

    default:
      return null;
  }
}